/*
   File: lxcn_test_lex.c
   Simple test program to check reading and searching lexica

   Copyright 2009 Radboud University of Nijmegen

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   CVS ID: "$Id$"
*/

/* standard includes */
#include <stdio.h>

/* libabase includes */
#include <abase_porting.h>
#include <abase_fileutil.h>
#include <abase_meminfo.h>
#include <abase_error.h>

/* local includes */
#include "lxcn_input.h"
#include "lxcn_lexicon.h"
#include "lxcn_search.h"

static void test_lexicon (Lexicon lex, char *input)
{ char *inp = input;
  char *end;
  LexemeType type;
  LexiconIterator iter;
  while (lxcn_is_white_space (*inp)) inp++;
  iter = lxcn_init_lexicon_match (inp, lex);
  while ((end = lxcn_next_lexicon_match (iter)) != NULL)
    { int entry_nr, nr_of_entries, ix;
      int *entries;
      char *lexeme;
      char *ptr;
      int penalty;
      lxcn_get_lexicon_match_info (iter, &entry_nr, &lexeme, &type, &penalty);
      nr_of_entries = lxcn_get_entries_from_nr (lex, entry_nr, &entries);
      abs_printf ("Matched lexeme type %d & string: ", type);
      for (ptr = input; ptr != end; ptr++) abs_printf ("%c", *ptr);
      abs_printf ("\n");
      abs_printf ("Matched entry nr %d, penalty %d, %d entries for lexeme: '",
		  entry_nr, penalty, nr_of_entries);
      lxcn_print_lexeme (lexeme, type);
      abs_printf ("'\n");
      for (ix = 0; ix < nr_of_entries; ix++)
        lxcn_print_entry (lex, entries, ix);
    };
  lxcn_finish_lexicon_match (iter);
}

int main (int argc, char **argv)
{ Lexicon lex;
  char lex_name[80];
  char buffer[256];
  if (argc > 1) sprintf (lex_name, "%s.blx", argv[1]);
  else sprintf (lex_name, "latin.blx");
  lxcn_init_char_tables (" \t\n\r",
			 "ABCDEFGHIJKLMNOPQRSTUVWXYZ", "abcdefghijklmnopqrstuvwxyz", NULL);
  lex = lxcn_read_lexicon (lex_name);
  if (lex != NULL)
  { while (fgets (buffer, 255, stdin) != NULL)
      test_lexicon (lex, buffer);
  }
  abs_report_meminfo ();
}
